#!/usr/bin/python -BO
import ttrr,numpy,scipy
# This file is a 3 dimensional example to show ttrr.
####################### design space #######################
dim = 3 # dimension of the design space
nelx = 3 # number of nodal points in x-direction
nely = nelx # number of nodal points in y-direction
nelz = nelx # number of nodal points in z-direction
# To let you easily change the design space, we use here a
# calculation to define the grid. The measurement unit is meter.
n = max(nelx,nely,nelz)
gx0 = 0.0
gx1 = (nelx-1)*1.0/(n-1)
gy0 = 0.0
gy1 = (nely-1)*1.0/(n-1)
gz0 = 0.0
gz1 = (nelz-1)*1.0/(n-1)
####################### bearings #######################
# We want to fix all nodes on the ground. We can describe these
# area (All nodes will become fixed bearings):
lagerbereiche = numpy.array([
    [ gx0 , gx1 , gy0 , gy1 , gz0 , gz0 ]
    ])
lagerbereiche[:,[0,2,4]] = lagerbereiche[:,[0,2,4]] - 1e-6
lagerbereiche[:,[1,3,5]] = lagerbereiche[:,[1,3,5]] + 1e-6
####################### get the parameter #######################
# Now we can get the parameter for the calculation from staebeknotenaufgitter3d.
# Here we can also use the optional parameter maxstablaenge to decide that
# bars longer than maxstablaenge should be ignored.
maxstablaenge = 3 # maximum allowed length of a bar
[knoten,staebe,lager] = ttrr.staebeknotenaufgitter3d(gx0,gx1,gy0,gy1,gz0,gz1,nelx,nely,nelz,lagerbereiche=lagerbereiche,maxstablaenge=maxstablaenge)
# We can look at the Parameter:
knoten
staebe
lager
(kn,kr) = knoten.shape
(sn,sr) = staebe.shape
(ln,) = lager.shape
# This means: We have kn=27 nodes and sn=274 bars. ln=27 node-directions are
# fixed. sr=2 is the number of nodes for a bar and kr=3 is the dimension of
# the design space.
####################### load cases #######################
# The loads are stored in a sparse matrix. For every node we have dim
# directions. So we have dim*kn rows and 2 cols for the 2 load cases.
kraefte = scipy.sparse.lil_matrix((dim*kn,2),dtype=numpy.float64) # 2 load cases
# The number k1 of a node can be calculated from its coordinates. The direction
# r1 is in z-direction. The magnitude -100000.0 descibes that the direction is
# the negative z-direction.
k1 = ttrr.knotennummervonkoordinaten(knoten,gx0,(gy0+gy1)/2,gz1) # position of the first force
r1 = dim*k1 # direction of the first force
kraefte[r1,0] = -100000.0 # Newton = ca. 1000 kg
# The same for the other load case:
k2 = ttrr.knotennummervonkoordinaten(knoten,gx1,(gy0+gy1)/2,gz1) # position of the second force
r2 = dim*k2 # direction of the second force
kraefte[r2,1] = 100000.0 # Newton = ca. 1000 kg
# We can look at the result:
print kraefte
####################### calculation #######################
# We need a few parameters for the calculation:
smax = 1 # meter^2
sigma = 4e8 # Pa = 100 MPa = 100 N / mm^2 (yield strength for some steel)
sigmamin = - sigma # compressive strength
sigmamax = sigma # tensile strength
dichte = 7e3 # kg / m^3 (density for some steel)
name="2_load_cases_3_dim"
# Now we can do the classical topology optimization:
[s,w,laengen] = ttrr.ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,opt=0)
# To calculate a redundant solution we need (This takes about 47 seconds on a
# computer with 3.6 GHz):
redundanz = 3
name="2_load_cases_3_dim_redundant"
[s,w,laengen] = ttrr.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=redundanz,opt=3,sort=1)
####################### visualisation #######################
# To get pictures, we can use ttrr_tools.py on the command line:
#  ttrr_tools.py
#  ttrr_tools.py -h
# We get quick and dirty pictures with gnuplot:
#  ttrr_tools.py --png_gnuplot -f 2_load_cases_3_dim*.tar.bz2
# Nice pictures are rendered by povray:
#  ttrr_tools.py --png_pov -f 2_load_cases_3_dim*.tar.bz2
# Because we are here viewing a 3 dimensional structure the 2 dimensional picture is insufficient. Therefore it is possible to get a movie animation:
#  trr_tools.py --ogv -f 2_load_cases_3_dim*.tar.bz2
# If you only want a small animation you can do one of the following:
#  trr_tools.py --apng -xm 320 -ym 240 -f 2_load_cases_3_dim*.tar.bz2
#  trr_tools.py --mng -xm 320 -ym 240 -f 2_load_cases_3_dim*.tar.bz2
#  trr_tools.py --gif -xm 320 -ym 240 -f 2_load_cases_3_dim*.tar.bz2
# You can also combine these options:
#   ttrr_tools.py -xm 320 -ym 240 -f 2_load_cases_3_dim*.tar.bz2 --png_pov --apng --mng --gif
#   ttrr_tools.py -xm 320 -ym 240 -f 2_load_cases_3_dim.tar.bz2 2_load_cases_3_dim_redundant.tar.bz2 --png_pov --apng --mng --gif
# The rendering by povray is time-consuming. Therefore it's done in parallel.
####################### conclusion #######################
# Many people are wondering because the asymmetrical result. At last it's a
# symmetric problem. But we have a linear problem. Therefore there possibly
# exists more than one edge of the polytope as an optimal solution --- the
# optimal value is the same. If we use an interior point algorithm we get a
# result between these edges.
#
# The interior point algorithm of glpk has a big problem with scaling.
# Therefore we do a small scaling per hand (Note: This is only a scaling
# factor. The topology is the same.)::
name="2_load_cases_3_dim_interior"
[s,w,laengen] = ttrr.ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,opt=1,scale_sigma=1e-8,scale_kraefte=1e-8)
# Now we can look at the result:
#    ttrr_tools.py -xm 320 -ym 240 -f 2_load_cases_3_dim_interior.tar.bz2 --png_pov --apng --mng --gif
# I hope you get the picture of the 2 edges of the polytope in this problem.
#
# Another interesting point is the fact that we see a 2 dimensional structure
# in a 3 dimension space. Because the loads are only acting in one plane this
# is possible. Sure a small perturbation would destroy this behavior. Therefore
# it is not a practical solution. If you want a practical solution think of
# robust.
name="2_load_cases_3_dim_robust"
[s,w,laengen] = ttrr.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,p=0.3,art=0,scale_sigma=1e-8,scale_kraefte=1e-8)
#    ttrr_tools.py -xm 320 -ym 240 -f 2_load_cases_3_dim_robust.tar.bz2 --png_pov --apng --mng --gif

# The redundant solution is for a redundancy of 1/3. Every color represent a part
# of the structure. Every part is allow to drop out. The other parts are able to
# support the given loads. The interesting point is that the red or blue structure
# are not connected.

# We can also compare the optimal values (This is the volume of the structures.):
#
# +------------------+--------------------+------------------+-----------------+
# | optimal solution | redundant solution | optimal solution | robust solution |
# +------------------+--------------------+------------------+-----------------+
# | simplex          | branch and bound   | interior point   | simplex         |
# +------------------+--------------------+------------------+-----------------+
# | 0.000875         | 0.001563           | 0.000875         | 0.001154        |
# +------------------+--------------------+------------------+-----------------+
#
# The optimal value of the scaled problem (which was used for interior problem)
# is 0.087500. If we unscale it with the above values we get the value in the
# table: 0.087500 * 1e-4 * 1e2
#
# The optimal value of the scaled problem (which was used for robust problem)
# is 0.115404. If we unscale it with the above values we get the value in the
# table: 0.115404 * 1e-4 * 1e2
