#!/usr/bin/python -BO
import ttrr,numpy,scipy # we need ttrr and other great packages
####################### info #######################
# This file shows you how ttrr is used. Some parts are unnecessary, but show the content/meaning of the concerning values or show same features/tools of ttrr. It's an example file!
#
# This file describes the prime example of a truss from:
# Aharon Ben-Tal and Arkadi Nemirovski. Robust truss topology design via semidefinite programming. SIAM J. Optim., 7(4):991-1016, November 1997. ISSN 1052-6234 (print), 1095-7189 (electronic). doi: 10.1137/S1052623495291951.
#
#        [ description ]            [ optimal solution ]       [  robust solution  ]
#
#                |
#                |
#        #1      3      5--         #------*------*            #------*------*
#                                           \                   \    / \    /
#                                            \                   \  /   \  /
#                                             \                   \/     \/
#                                              \                  /\     /\
#                                               \                /  \   /  \
#                                                \              /    \ /    \
#        #0    --2      4           #------*------*            #------*------*
#                       |
#                       |
#
####################### design space #######################
dim = 2 # dimension of the design space
nelx = 3 # number of nodes in x direction
nely = 2 # number of nodes in y direction
kn = nelx*nely # number or total nodes
# definition of the grid
gx0 = 0.0 # meter
gx1 = 2.0 # meter
gy0 = 0.0 # meter
gy1 = 1.0 # meter
# Now we have a design space of nelx*nely=3*2=6 nodes. The design space is a rectangle of the points (x,y) with gx0 <= x <= gx1 and gy0 <= y <= gy1. We need the coordinates of alle nodes:
knoten = numpy.array([[ 0.0 , 0.0 ], # node 0 with the coordinates (0.0,0.0) = knoten[0,:]
                      [ 0.0 , 1.0 ], # node 1 with the coordinates (0.0,1.0) = knoten[1,:]
                      [ 1.0 , 0.0 ], # ...
                      [ 1.0 , 1.0 ],
                      [ 2.0 , 0.0 ],
                      [ 2.0 , 1.0 ]],dtype=numpy.float64) # the coordinates are float64 (double)
# We need a description of all trusses: (from every node to every other one)
staebe = numpy.array([[0,2], # truss from node 0 with knoten[0,:] to node 2 with knoten[2,:]
                      [0,1], # truss from node 0 with knoten[0,:] to node 1 with knoten[1,:]
                      [0,3], # ...
                      [0,5],
                      [1,3],
                      [2,4],
                      [1,2],
                      [2,3],
                      [2,5],
                      [3,5],
                      [1,4],
                      [3,4],
                      [4,5]],dtype=numpy.uint16) # the number of nodes are uint16 (16-bit int > 0)
# We do not need a truss from 0 to 4 or from 1 to 5, because it is the same as from 0 to 2 and from 2 to 4 or from 1 to 3 and from 3 to 5, respectivly.
# We can get this from, too: (This is handy if you have more than 6 nodes!)
[knoten,staebe] = ttrr.staebeknotenaufgitter2d(gx0,gx1,gy0,gy1,nelx,nely)
####################### bearings #######################
# We want to fix the node 0 and 1 in every direction:
#   0 for node 0 in x direction
#   1 for node 0 in y direction
#   2 for node 1 in x direction
#   3 for node 1 in y direction
lager = numpy.array([0,1,2,3],dtype=numpy.uint16)
# The truss from node 0 to node 1 is on both ends fixed. So this truss do not make sense. To reduce the number of design variables we can do it without this truss:
[knoten,staebe] = ttrr.staebeknotenaufgitter2d(gx0,gx1,gy0,gy1,nelx,nely,lager=lager)
####################### load cases #######################
# We have only 1 load case, but with "some" forces on different nodes:
kraefte = scipy.sparse.lil_matrix((dim*kn,1),dtype=numpy.float64)
kraefte[4,0] = -10000 # force in node 2 in x direction means 4 in the load case 0 with a value of -10000 N; this means a force in -x direction with the magnitude 10000 N in node 2
kraefte[7,0] = 10000 # force in node 3 in y direction means 7 in the load case 0 with a value of 10000 N; this means a force in y direction with the magnitude 10000 N in node 3
kraefte[9,0] = -10000 # force in -y direction with the magnitude 10000 N in node 4
kraefte[10,0] = 10000 # force in x direction with the magnitude 10000 N in node 5
# The magnitude of each force is only a scalar factor in the optimization problem. The commanding spot is the proportion of the forces among each other. The same holds for the material parameters.
####################### calculation #######################
# We need a few parameters for the calculation:
smax = 1 # the maximum allowed cross section in meter (here: not realy a bound)
name = "ben-tal_nemirovski_prime_example" # a name for the output
# Some material parameter: (The compressive strength sigmamin and the tensile strength sigmamax are only scalar factors in the calculation. The density dichte is only used to show the mass of the result.)
sigma = 1e8 # Pa = 100 MPa = 100 N / mm^2 (yield strength for aluminium)
sigmamin = - sigma # compressive strength
sigmamax = sigma # tensile strength
dichte = 2.7e3 # kg / m^3 (density for aluminium)
# Now we can calculate the result:
[s,w,laengen] = ttrr.ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte)
# As in "Aharon Ben-Tal and Arkadi Nemirovski. Robust truss topology design via semidefinite programming. SIAM J. Optim., 7(4):991-1016, November 1997. ISSN 1052-6234 (print), 1095-7189 (electronic). doi: 10.1137/S1052623495291951." we can get a robust solution; but here with a linear program as described in "Mohr, Daniel P. ; Stein, Ina ; Matzies, Thomas ; Knapek, Christina A.: Robust Topology Optimization of Truss with regard to Volume. In: arXiv - Mathematics, Optimization and Control (2011). http://arxiv.org/abs/1109.3782":
name = "ben-tal_nemirovski_prime_example_robust" # a name for the output
[s,w,laengen] = ttrr.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,p=0.1,art=0)
# More explicitly we can calculate (or even define) the perturbed forces
kraefte_gestoert = ttrr.berechne_gestoerte_kraefte(kraefte,dim,0.1,art=0)
# and do the computation with these:
[s,w,laengen] = ttrr.ttrr(dim,knoten,staebe,lager,kraefte_gestoert,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte)
# For the same problem we can calculate a (1/2)-redundant solution as described in "Mohr, Daniel P.: Redundante Topologieoptimierung. Neubiberg, Universitaet der Bundeswehr Muenchen, Fakultaet fuer Luft- und Raumfahrttechnik, Diss., Dezember 2011. http://nbn-resolving.de/urn/resolver.pl?urn=urn:nbn:de:bvb:706-2664":
name = "ben-tal_nemirovski_prime_example_redundant" # a name for the output
[s,w,laengen] = ttrr.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=2,opt=3)
####################### visualisation #######################
# To get pictures, we can use ttrr_tools.py on the command line:
# ttrr_tools.py --png_gnuplot -f ben-tal_nemirovski_prime_example.tar.bz2 ben-tal_nemirovski_prime_example_robust.tar.bz2 ben-tal_nemirovski_prime_example_redundant.tar.bz2
####################### conclusion #######################
# The degree of freedom for this problem is very small. This means: The set of admissible solutions is small. So the optimization routine has only a small choice. The consequence is that in particular the result of the redundant solution is poor. But this file shows the functionality of ttrr and also the basic concept of robustness and redundancy. I hope you get the point.
