"""TrussTopologyRobustRedundant

.. contents::

description
===========
ttrr is a software package to do topology optimization of a truss
with respect to robustness and redundancy --- see literatur.

There are 2 parts of ttrr:

* library:
    You can use :py:mod:`ttrr` as a library to import to your own python-code.
    To calculate trusses :py:mod:`ttrr` needs :program:`ttrr_calculate` resp. ttrr_calculate.cc

* command line tool:
    you can use :py:mod:`ttrr_tools.py` as a command line tool.
    for detail::

        ttrr_tools.py -h

functions to include from ttrr resp. ttrr.py: (in alphabetical order)
    * :py:func:`berechne_gestoerte_kraefte`
    * :py:func:`knotennummervonkoordinaten`
    * :py:func:`staebeknotenaufgitter2d`
    * :py:func:`staebeknotenaufgitter3d`
    * :py:func:`.ttrr`
    * :py:func:`ttrr_redundanz`
    * :py:func:`ttrr_robust`


typical parameters
==================
dim : integer
      dimension 2 or 3 of the design space

knoten : numpy.array
         is a matrix to represent all nodal points with:

         * knoten(k,:) are the nodal points.
         * knoten(:,1) are the x-coordinates of the nodal points.
         * knoten(:,2) are the y-coordinates of the nodal points.
         * knoten(:,3) are the z-coordinates of the nodal points. (mostly optional)

staebe : numpy.array
         is a vector to represent all trusses:

         * staebe(s,:) are numbers of the trusses.
         * staebe(:,1) is the number of the first nodal point.
         * staebe(:,2) is the number of the second nodal point.

lager : numpy.array
        are the numbers of the nodal points as for the coordinate direction
        which are fixed.

kraefte : scipy.sparse.lil_matrix
          are the load forces.

          kraefte(:,k) are the forces for the k-th load case

smax : float
       is the maximum allowed cross section of a truss

opt : integer
      choose the optimization routine: (optional)

      * 0 glp_simplex (default, iff redundanz == 0)
      * 1 glp_interior (only for redundanz == 0)
      * 2-5 glp_mip (for redundanz != 0)

        * 2 do not search greedy-solution (default, for redundanz != 0)
        * 3 search greedy-solution and go on
        * 4 search greedy-solution, write greedy solution and go on
        * 5 search greedy-solution and stop


functions
=========
.. currentmodule:: ttrr
.. autofunction:: berechne_gestoerte_kraefte
.. autofunction:: knotennummervonkoordinaten
.. autofunction:: staebeknotenaufgitter2d
.. autofunction:: staebeknotenaufgitter3d
.. autofunction:: ttrr
.. autofunction:: ttrr_redundanz
.. autofunction:: ttrr_robust


dependencies
============
Dependencies for ttrr.py and ttrr_tools.py:
  * Python http://www.python.org/
  * NumPy http://numpy.scipy.org/ http://pypi.python.org/pypi/numpy
  * SciPy http://www.scipy.org/ http://pypi.python.org/pypi/scipy
  
    * and also for NumPy/SciPy: BLAS and LAPACK libraries
  
Dependencies for ttrr and ttrr.cc, respectively:

  * GLPK http://www.gnu.org/software/glpk/

Runtime dependencies for ttrr_tools.py to get graphical output (picture, movie,
animation) you need some of the following programs:

  * gnuplot http://www.gnuplot.info/
  * POV-Ray http://www.povray.org/
  * png2yuv from MJPEG http://mjpeg.sourceforge.net/
  * ffmpeg2theora http://www.v2v.cc/~j/ffmpeg2theora/
  * png2theora from libtheora http://www.theora.org
  * FFmpeg http://ffmpeg.org/
  * APNG Assembler http://sourceforge.net/projects/apngasm/
  * convert from ImageMagick http://www.imagemagick.org/

At least for pictures you need gnuplot and/or POV-Ray. The frames for all movie and animation output are rendered with POV-Ray.


References:
===========

* Mohr, Daniel P. ; Stein, Ina ; Matzies, Thomas ; Knapek, Christina A.:
  Robust Topology Optimization of Truss with regard to Volume.
  In: arXiv - Mathematics, Optimization and Control (2011).
  http://arxiv.org/abs/1109.3782v1

* Mohr, Daniel P.: Redundante Topologieoptimierung.
  Neubiberg, Universitaet der Bundeswehr Muenchen, Fakultaet fuer Luft- und Raumfahrttechnik, Diss., Dezember 2011.
  http://nbn-resolving.de/urn/resolver.pl?urn=urn:nbn:de:bvb:706-2664


copyright + license
===================
Author: Daniel Mohr.

Date: 2012-02-22 (last change).

License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

Copyright (C) 2009, 2010, 2011, 2012 Daniel Mohr
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 3 of
 the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, see
 http://www.gnu.org/licenses/
"""

from .intern.staebeknotenaufgitter2d import *
from .intern.staebeknotenaufgitter3d import *
from .intern.berechne_gestoerte_kraefte import *
from .intern.knotennummervonkoordinaten import *
from .intern.ttrr import *
from .intern.ttrr_tools import ggT,iseven,isodd

__all__ = ["berechne_gestoerte_kraefte","staebeknotenaufgitter2d","staebeknotenaufgitter3d","ttrr","ttrr_robust","ttrr_redundanz","knotennummervonkoordinaten"]
