import numpy
from ttrr_tools import ggT

def staebeknotenaufgitter2d(gx0,gx1,gy0,gy1,nelx,nely,lager=0):
    """[knoten,staebe]=staebeknotenaufgitter(gx0,gx1,gy0,gy1,nelx,nely,lager=0)
    
    creates all bars and nodes which make sense. gx0, gx1, gy0 and gy1 describes
    the edges (gx0,gy0), (gx0,gy1), (gx1,gy0) and (gx1,gy1) of the rectangular
    design space. In this rectangle it will be nelx*nely nodes calculated. With
    the assumption of an equidistant grid all meaningful bars are calculated.
    Bars which can be represent of others are ignored. For example a bar from
    (0,0) to (0,1) is ignored, if bars from (0,0) to (0,0.5) and from (0,0.5)
    to (0,1) exist.

    Parameters:
        gx0 : float
              lower bound of the x-cooredinates to describe the rectangle
        gx1 : float
              upper bound of the x-cooredinates to describe the rectangle
        gy0 : float
              lower bound of the y-cooredinates to describe the rectangle
        gy0 : float
              upper bound of the y-cooredinates to describe the rectangle
        nelx : integer
               is the number of nodes in x-direction
        nely : integer
               is the number of nodes in y-direction
        lager : numpy.array
                are the numbers of the nodal points as for the coordinate
                direction which are fixed. (optional)
                If denoted bars fixed on both ends will be ignored.

    Returns:
        knoten : numpy.array
                 is a matrix to represent all nodal points with:

                 * knoten(k,:) are the nodal points.
                 * knoten(:,1) are the x-coordinates of the nodal points.
                 * knoten(:,2) are the y-coordinates of the nodal points.
                 * knoten(:,3) are the z-coordinates of the nodal points. (mostly optional)
        staebe : numpy.array
                 is a vector to represent all trusses:

                 * staebe(s,:) are numbers of the trusses.
                 * staebe(:,1) is the number of the first nodal point.
                 * staebe(:,2) is the number of the second nodal point.
    
    copyright + license

    Author: Daniel Mohr.

    Date: 2010-09-09, 2011-06-24, 2012-01-24 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2009, 2010, 2011, 2012 Daniel Mohr
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 3 of
     the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, see
     http://www.gnu.org/licenses/"""
    gx0 = numpy.float64(gx0)
    gx1 = numpy.float64(gx1)
    gy0 = numpy.float64(gy0)
    gy1 = numpy.float64(gy1)
    kn = nelx*nely # Anzahl der Knoten
    knoten = numpy.zeros((kn,2),dtype=numpy.float64)
    fx = (gx1-gx0)/(nelx-1)
    fy = (gy1-gy0)/(nely-1)
    for elx in range(0,nelx):
        for ely in range(0,nely):
            k = ely+nely*elx # Knotennummer
            knoten[k,0] = elx * fx # x-Koordinate
            knoten[k,1] = ely * fy # y-Koordinate
    staebe = numpy.zeros(((nelx*nely)*(nelx*nely+1)/2,2),dtype=numpy.uint16)
    sn = 0
    for elx in range(0,nelx):
        for ely in range(0,nely):
            k1 = ely+nely*elx # Knotennummer
            rx = 1
            ry = 0
            if elx + rx < nelx:
                k2 = (ely+ry)+nely*(elx+rx) # andere Knotennummer
                if (k1 < k2):
                    staebe[sn,0] = k1
                    staebe[sn,1] = k2
                else:
                    staebe[sn,0] = k2
                    staebe[sn,1] = k1
                sn = sn + 1
            for rx in range(-elx,-elx+nelx):
                for ry in range(1,-ely+nely):
                    if rx == 0:
                        if ry == 1:
                            k2 = (ely+ry)+nely*(elx+rx) # andere Knotennummer
                            if (k1 < k2):
                                staebe[sn,0] = k1
                                staebe[sn,1] = k2
                            else:
                                staebe[sn,0] = k2
                                staebe[sn,1] = k1
                            sn = sn + 1
                    else:
                        if ggT(rx,ry) == 1:
                            #print "sn",sn,(nelx*nely)*(nelx*nely+1)/2
                            k2 = (ely+ry)+nely*(elx+rx) # andere Knotennummer
                            if (k1 < k2):
                                staebe[sn,0] = k1
                                staebe[sn,1] = k2
                            else:
                                staebe[sn,0] = k2
                                staebe[sn,1] = k1
                            sn = sn + 1
                # for ry ende
            # for rx ende
        # for ely ende
    # for elx ende
    staebe = staebe[0:sn,]
    if numpy.isscalar(lager) == False:
        staebe2 = numpy.zeros((sn,2),dtype=numpy.uint16);
        sn2 = 0
        (ln,) = lager.shape
        print sn," potenzielle Staebe gefunden; beruecksichtige",ln,"Lager"
        for i in range(0,sn):
            kommtvor0 = False
            kommtvor1 = False
            for j in range(0,ln):
                if numpy.floor(lager[j]/3.0)==staebe[i,0]:
                    kommtvor0 = True
                if numpy.floor(lager[j]/3.0)==staebe[i,1]:
                    kommtvor1 = True
            if (kommtvor0 == False) or (kommtvor1 == False):
                staebe2[sn2,0] = staebe[i,0]
                staebe2[sn2,1] = staebe[i,1]
                sn2 = sn2 + 1
        print "es bleiben",sn2,"Staebe, die nicht nur gelagert sind"
        staebe = staebe2[0:sn2,]
    return [knoten,staebe]
