import numpy
from ttrr_tools import ggT

def staebeknotenaufgitter3d(gx0,gx1,gy0,gy1,gz0,gz1,nelx,nely,nelz,lager=0,bereicheohneknoten=0,lagerbereiche=0,maxstablaenge=-1):
    """[knoten,staebe]=staebeknotenaufgitter3d(gx0,gx1,gy0,gy1,gz0,gz1,nelx,
                       nely,nelz,lager=0,bereicheohneknoten=0,maxstablaenge=-1)
    [knoten,staebe,lager]=staebeknotenaufgitter3d(gx0,gx1,gy0,gy1,gz0,gz1,nelx,
                          nely,nelz,bereicheohneknoten=0,lagerbereiche=0,
                          maxstablaenge=-1)

    creates all bars and nodes which make sense. If lagerbereiche is denoted
    the fixed nodes are also calculated and returned.
    gx0, gx1, gy0, gy1, gz0 and gz1 describes the edges
    (gx0,gy0,gz0), (gx0,gy0,gz1), (gx0,gy1,gz0), (gx0,gy1,gz1)
    (gx1,gy0,gz0), (gx1,gy0,gz1), (gx1,gy1,gz0) and (gx1,gy1,gz1)
    of the cuboid-shaped design space. In this cuboid it will be nelx*nely*nelz
    nodes calculated. With the assumption of an equidistant grid all meaningful
    bars are calculated. Bars which can be represent of others are ignored. For
    example a bar from (0,0,0) to (0,0,2) is ignored, if bars from (0,0,0) to
    (0,0,1) and from (0,0,1) to (0,0,2) exist.

    Parameters:
        gx0 : float
              lower bound of the x-cooredinates to describe the rectangle
        gx1 : float
              upper bound of the x-cooredinates to describe the rectangle
        gy0 : float
              lower bound of the y-cooredinates to describe the rectangle
        gy0 : float
              upper bound of the y-cooredinates to describe the rectangle
        gz0 : float
              lower bound of the z-cooredinates to describe the rectangle
        gz0 : float
              upper bound of the z-cooredinates to describe the rectangle
        nelx : integer
               is the number of nodes in x-direction
        nely : integer
               is the number of nodes in y-direction
        nelz : integer
               is the number of nodes in z-direction
        lager : numpy.array
                are the numbers of the nodal points as for the coordinate
                direction which are fixed. (optional)
                If denoted bars fixed on both ends will be ignored.
        bereicheohneknoten : numpy.array
                             is a matrix describing areas without nodes (optional)
                             In every row are 6 values ax, bx, ay, by, az, bz.
                             Nodes with coordinate (kx,ky,kz) and
                             (ax<=kx<=bx and ay<=ky<=by and az<=kz<=bz) will be
                             ignored for the bars. Theses nodes are in knoten. But
                             no bar will be connect to these nodes.
        lagerbereiche : numpy.array
                        is a matrix describing areas where the nodes are
                        fixed. (optional)
                        In every row are 6 values ax, bx, ay, by, az, bz.
                        Nodes with coordinate (kx,ky,kz) and
                        (ax<=kx<=bx and ay<=ky<=by and az<=kz<=bz) are
                        considered as fixed nodes.
                        If lagerbereiche is denoted also lager will be returned.
                        The paarameter lager will be ignored. But the affected
                        bars will be ignored.
        maxstablaenge : float
                        Bars which are longer than maxstablaenge will be
                        ignored.

    Returns:
        knoten : numpy.array
                 is a matrix to represent all nodal points with:

                 * knoten(k,:) are the nodal points.
                 * knoten(:,1) are the x-coordinates of the nodal points.
                 * knoten(:,2) are the y-coordinates of the nodal points.
                 * knoten(:,3) are the z-coordinates of the nodal points.
        staebe : numpy.array
                 is a vector to represent all trusses:

                 * staebe(s,:) are numbers of the trusses.
                 * staebe(:,1) is the number of the first nodal point.
                 * staebe(:,2) is the number of the second nodal point.
        lager : numpy.array
                are the numbers of the nodal points as for the coordinate
                direction which are fixed. (optional)
                It will will be returned only if lagerbereiche is denoted.

    copyright + license

    Author: Daniel Mohr.

    Date: 2011-06-24, 2012-01-24, 2012-01-25 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2011, 2012 Daniel Mohr
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 3 of
     the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, see
     http://www.gnu.org/licenses/"""
    gx0 = numpy.float64(gx0)
    gx1 = numpy.float64(gx1)
    gy0 = numpy.float64(gy0)
    gy1 = numpy.float64(gy1)
    gz0 = numpy.float64(gz0)
    gz1 = numpy.float64(gz1)
    kn = nelx*nely*nelz
    knoten = numpy.zeros((kn,3),dtype=numpy.float64)
    fx = (gx1-gx0)/(nelx-1)
    fy = (gy1-gy0)/(nely-1)
    fz = (gz1-gz0)/(nelz-1)
    for elx in range(0,nelx):
        for ely in range(0,nely):
            for elz in range(0,nelz):
                k = ely+nely*elx+nelx*nely*elz # Knotennummer
                knoten[k,0] = gx0 + elx * fx # x-Koordinate
                knoten[k,1] = gy0 + ely * fy # y-Koordinate
                knoten[k,2] = gz0 + elz * fz # z-Koordinate
    (kn,dim) = knoten.shape
    if numpy.isscalar(lagerbereiche) == False:
        # lager wurde ueber lagerbereiche angegeben
        (lbn,t) = lagerbereiche.shape
        print('lager wird aus %d Zeilen von lagerbereiche erstellt' % lbn)
        lager=numpy.arange(0,0)
        gelagerteknoten = 0
        for k in range(0,kn):
            for b in range(0,lbn):
                if (((lagerbereiche[b,0] <= knoten[k,0]) and
                     (knoten[k,0] <= lagerbereiche[b,1])) and
                    ((lagerbereiche[b,2] <= knoten[k,1]) and
                     (knoten[k,1] <= lagerbereiche[b,3])) and
                    ((lagerbereiche[b,4] <= knoten[k,2]) and
                     (knoten[k,2] <= lagerbereiche[b,5]))):
                    for i in range(0,dim):
                        lager = numpy.append(lager,dim*k+i)
                    gelagerteknoten = gelagerteknoten + 1
        (ln,) = lager.shape
        lager = sorted(lager)
        i = 1
        entferntelager = 0
        while i < ln:
            if lager[i] == lager[i-1]:
                lager = numpy.delete(lager,i)
                (ln,) = lager.shape
                entferntelager = entferntelager + 1
            else:
                i = i + 1
        gelagerteknoten = gelagerteknoten - entferntelager/3
        lagerlist = lager
        lager = numpy.arange(0,ln)
        for i in range(0,ln):
            lager[i] = lagerlist[i]
        print('%d Knoten sind gelagert; %d Lager' % (gelagerteknoten,ln))
    staebe = numpy.zeros((nelx*nely*nelz*(nelx*nely*nelz+1)/2,2),dtype=numpy.uint16);
    sn = 0
    for elx in range(0,nelx):
        for ely in range(0,nely):
            for elz in range(0,nelz):
                k1 = ely + nely*elx + nelx*nely*elz # Knotennummer
                rx = 0; ry = 0; rz = 1;
                if elz+rz < nelz:
                    k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz); # andere Knotennummer
                    staebe[sn,0] = k1;
                    staebe[sn,1] = k2;
                    sn = sn + 1;
                rx = 0; ry = 1; rz = 0;
                if ely+ry < nely:
                    k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz); # andere Knotennummer
                    staebe[sn,0] = k1;
                    staebe[sn,1] = k2;
                    sn = sn + 1;
                rx = 1; ry = 0; rz = 0;
                if elx+rx < nelx:
                    k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz); # andere Knotennummer
                    staebe[sn,0] = k1;
                    staebe[sn,1] = k2;
                    sn = sn + 1;
                rz = 0;
                for rx in range(-elx,-elx+nelx):
                    for ry in range(1,-ely+nely):
                        if rx != 0:
                            if ggT(rx,ry) == 1:
                                k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz) # andere Knotennummer
                                if k1 < k2:
                                    staebe[sn,0] = k1;
                                    staebe[sn,1] = k2;
                                else:
                                    staebe[sn,0] = k2;
                                    staebe[sn,1] = k1;
                                sn = sn + 1;
                   # for ry end
                # for ry end
                ry = 0
                for rx in range(-elx,-elx+nelx):
                    for rz in range(1,-elz+nelz):
                        if rx != 0:
                            if ggT(rx,rz) == 1:
                                k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz); # andere Knotennummer
                                if k1 < k2:
                                    staebe[sn,0] = k1;
                                    staebe[sn,1] = k2;
                                else:
                                    staebe[sn,0] = k2;
                                    staebe[sn,1] = k1;
                                sn = sn + 1;
                    # for rz end
                # for rx end
                rx = 0
                for ry in range(-ely,-ely+nely):
                    for rz in range(1,-elz+nelz):
                        if ry != 0:
                            if ggT(ry,rz) == 1:
                                k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz) # andere Knotennummer
                                if k1 < k2:
                                    staebe[sn,0] = k1;
                                    staebe[sn,1] = k2;
                                else:
                                    staebe[sn,0] = k2;
                                    staebe[sn,1] = k1;
                                sn = sn + 1
                    # for rz end
                # for ry end
                for rx in range(-elx,-elx+nelx):
                    for ry in range(-ely,-ely+nely):
                        for rz in range(1,-elz+nelz):
                            if (rx != 0) and (ry != 0):
                                if ggT(ggT(rx,ry),ggT(rx,rz)) == 1:
                                    k2 = (ely+ry) + nely*(elx+rx) + nelx*nely*(elz+rz) # andere Knotennummer
                                    if k1 < k2:
                                        staebe[sn,0] = k1
                                        staebe[sn,1] = k2
                                    else:
                                        staebe[sn,0] = k2
                                        staebe[sn,1] = k1
                                    sn = sn + 1
                        # for rz end
                    # for rz end
                # for rx end
            # for elz ende
        # for ely ende
    # for elx ende
    staebe = staebe[0:sn,]
    if numpy.isscalar(bereicheohneknoten) == False:
        (bn,t) = bereicheohneknoten.shape
        (sn,t) = staebe.shape
        print('%d Bereiche ohne Knoten als Stabende werden beruecksichtigt' % bn)
        print('bisher %d Staebe' % sn)
        i = 0
        while i < sn:
            stabendeineinembereich = False
            for b in range(0,bn):
                for j in range(0,1):
                    if (((bereicheohneknoten[b,0] <= knoten[staebe[i,j],0]) and
                         (knoten[staebe[i,j],0] <= bereicheohneknoten[b,1])) and
                        ((bereicheohneknoten[b,2] <= knoten[staebe[i,j],1]) and
                         (knoten[staebe[i,j],1] <= bereicheohneknoten[b,3])) and
                        ((bereicheohneknoten[b,4] <= knoten[staebe[i,j],2]) and
                         (knoten[staebe[i,j],2] <= bereicheohneknoten[b,5]))):
                        stabendeineinembereich = True
            if stabendeineinembereich:
                staebe = numpy.delete(staebe,i,axis=0)
                (sn,t) = staebe.shape
            else:
                i = i + 1
        print('es bleiben %d Staebe' % sn)
    # end: if numpy.isscalar(bereicheohneknoten) == False:
    if numpy.isscalar(lager) == False:
        (ln,) = lager.shape
        print('%d potenzielle Staebe gefunden; beruecksichtige %d Lager' % (sn,ln))
        (sn,t) = staebe.shape
        i = 0
        while i < sn:
            kommtvor0 = False
            kommtvor1 = False
            for j in range(0,ln):
                if numpy.floor(lager[j]/3.0) == staebe[i,0]:
                    kommtvor0 = True
                if numpy.floor(lager[j]/3.0) == staebe[i,1]:
                    kommtvor1 = True
            if kommtvor0 and kommtvor1:
                staebe = numpy.delete(staebe,i,axis=0)
                (sn,t) = staebe.shape
            else:
                i = i + 1
        print('es bleiben %d Staebe, die nicht nur gelagert sind' % sn)
    if maxstablaenge > 0:
        print('%d potenzielle Staebe; beruecksichtige maximale Stablaenge %f' % (sn,maxstablaenge))
        (sn,t) = staebe.shape
        s = 0
        while s < sn:
            a=staebe[s,0]
            b=staebe[s,1]
            ax=knoten[a,0]
            ay=knoten[a,1]
            bx=knoten[b,0]
            by=knoten[b,1]
            dx=ax-bx
            dy=ay-by
            az=knoten[a,2]
            bz=knoten[b,2]
            dz=az-bz
            laenge = numpy.sqrt(numpy.square(dx)+numpy.square(dy)+numpy.square(dz))
            if laenge > maxstablaenge:
                staebe = numpy.delete(staebe,s,axis=0)
                (sn,t) = staebe.shape
            else:
                s = s + 1
        print('es bleiben %d Staebe, die kuerzer sind als %f' % (sn,maxstablaenge))
    # end: if numpy.isscalar(lager) == False:
    if numpy.isscalar(lagerbereiche):
        ret = [knoten,staebe]
    else:
        ret = [knoten,staebe,lager]
    return ret
