from ttrr_write import *
from ttrr_read import *
from ttrr_tools import *
from berechne_gestoerte_kraefte import *

def ttrr(dim,knoten,staebe,lager,kraefte,smax,name='test',sigmamin=-1e8,sigmamax=1e8,dichte=2.7e3,redundanz=0,opt=0,ungestoerte_kraefte=0,sort=0,zlb=0,zub=0,scale_sigma=-1.0,scale_kraefte=-1.0):
    """[s,w,laengen] = ttrr(dim,knoten,staebe,lager,kraefte,smax,name='test',
                       sigmamin=-1e8,sigmamax=1e8,dichte=2.7e3,redundanz=0,
                       opt=0,ungestoerte_kraefte=0,sort=0,zlb=0,zub=0,
                       scale_sigma=-1,scale_kraefte=-1)

        Parameters:

            dim : integer
                  dimension 2 or 3 of the design space
            knoten : numpy.array
                     is a matrix to represent all nodal points with:

                     * knoten(k,:) are the nodal points.
                     * knoten(:,1) are the x-coordinates of the nodal points.
                     * knoten(:,2) are the y-coordinates of the nodal points.
                     * knoten(:,3) are the z-coordinates of the nodal points.
                                   (optional for dim = 2)
            staebe : numpy.array
                     is a vector to represent all trusses:

                     * staebe(s,:) are numbers of the trusses.
                     * staebe(:,1) is the number of the first nodal point.
                     * staebe(:,2) is the number of the second nodal point.
            lager : numpy.array
                    are the numbers of the nodal points as for the coordinate
                    direction which are fixed.
            kraefte : scipy.sparse.lil_matrix
                      are the load forces.

                      kraefte(:,k) are the forces for the k-th load case
            smax : float
                   is the maximal allowed cross section of a bar. If you do not
                   need it, set it to a huge value --- e. g. 1.
            name : string
                   is a name for the problem. The result is saved in name.tar.bz2
                   in the current working directory.
            sigmamin : float
                       lower limit for the allowed stress,
                       e.g. compressive strength (e.g. -1e8 Pa for aluminium or -4e8 Pa
                       for some steel)
            sigmamax : float
                       upper limit for the allowed stress,
                       e.g. tensile strength (e.g. 1e8 Pa for aluminium or 4e8 Pa for
                       some steel)
            dichte : float
                     density (e.g. 2.7e3 kg/m^3 for aluminium or 7e3 kg/m^3 for some
                     steel)
            redundanz : integer
                        describes the redundancy. If redundanz != 0 this function
                        is not the right one. :py:func:`ttrr_redundanz` will be called.
            opt : integer
                  choose the optimization routine: (optional)

                  * 0 glp_simplex (default, iff redundanz == 0)
                  * 1 glp_interior (only for redundanz == 0)
                  * 2-5 glp_mip (only for redundanz != 0, therefore not in this
                        function) see :py:func:`ttrr_redundanz`
            ungestoerte_kraefte : numpy.array
                                  is not used. It will be saved to the problem data.
            sort : integer
                   describes a feature: (optional)

                   * 0 not used (default)
                   * 1 additional constraints will be added to the optimization
                       problem. This is only useful for redundanz != 0.
                       see :py:func:`ttrr_redundanz`
            zlb : float
                  is a lower bound for the objective function. As we are minimizing
                  the result must not the best solution. Only values greater zero
                  are considered. (optional)
            zub : float
                  is a upper bound for the objective function. As we are minimizing
                  the optimization problem may be not solvable. Only values greater
                  zero are considered. (optional)
            scale_sigma : float
                  scales sigmamin and sigmamax for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value
            scale_kraefte : float
                  scales kraefte for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value

    copyright + license

    Author: Daniel Mohr.

    Date: 2012-02-19 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2012 Daniel Mohr
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 3 of
     the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, see
     http://www.gnu.org/licenses/"""
    if redundanz != 0:
        return ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=redundanz,opt=opt,ungestoerte_kraefte=ungestoerte_kraefte,sort=sort,zlb=zlb,zub=zub,scale_sigma=scale_sigma,scale_kraefte=scale_kraefte)
    print('###### start ttrr ######')
    # schreibe Problem
    tmpdir = schreibe_problem(dim,knoten,staebe,lager,kraefte,smax,name,redundanz,opt,ungestoerte_kraefte,sort,sigmamin,sigmamax,dichte,zlb,zub,scale_sigma,scale_kraefte)
    print('tmpdir: %s' % tmpdir)
    # speichere Problem
    speichere_problem(name,tmpdir)
    # lasse Problem loesen
    lasse_problem_loesen(tmpdir)
    speichere_problem(name,tmpdir)
    # lese Problem
    s=lese_double_vektor(tmpdir,"s")
    w=lese_double_vektor(tmpdir,"w")
    laengen=lese_double_vektor(tmpdir,"laengen")
    for d in os.listdir(tmpdir):
        os.unlink('%s/%s' % (tmpdir,d))
    os.rmdir(tmpdir)
    print('###### end ttrr ######')
    return [s,w,laengen]

def ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name='test',sigmamin=-1e8,sigmamax=1e8,dichte=2.7e3,redundanz=0,opt=0,ungestoerte_kraefte=0,p=0.1,art=0,sort=0,zlb=0,zub=0,scale_sigma=-1.0,scale_kraefte=-1.0):
    """[s,w,exitflag,zeit,laengen] = ttrr_robust(dim,knoten,staebe,lager,
                                     kraefte,smax,name='test',sigmamin=-1e8,
                                     sigmamax=1e8,dichte=2.7e3,redundanz=0,
                                     opt=0,ungestoerte_kraefte=0,p=0.1,art=0,
                                     sort=0,zlb=0,zub=0,
                                     scale_sigma=-1,scale_kraefte=-1)


        Parameters:

            dim : integer
                  dimension 2 or 3 of the design space
            knoten : numpy.array
                     is a matrix to represent all nodal points with:

                     * knoten(k,:) are the nodal points.
                     * knoten(:,1) are the x-coordinates of the nodal points.
                     * knoten(:,2) are the y-coordinates of the nodal points.
                     * knoten(:,3) are the z-coordinates of the nodal points.
                                   (optional for dim = 2)
            staebe : numpy.array
                     is a vector to represent all trusses:

                     * staebe(s,:) are numbers of the trusses.
                     * staebe(:,1) is the number of the first nodal point.
                     * staebe(:,2) is the number of the second nodal point.
            lager : numpy.array
                    are the numbers of the nodal points as for the coordinate
                    direction which are fixed.
            kraefte : scipy.sparse.lil_matrix
                      are the load forces.

                      kraefte(:,k) are the forces for the k-th load case
            smax : float
                   is the maximal allowed cross section of a bar. If you do not
                   need it, set it to a huge value --- e. g. 1.
                   It is nessecary for the interaction of the integer values
                   and the cross section values (for redundanz != 0).
            name : string
                   is a name for the problem. The result is saved in name.tar.bz2
                   in the current working directory.
            sigmamin : float
                       lower limit for the allowed stress,
                       e.g. compressive strength (e.g. -1e8 Pa for aluminium or -4e8 Pa
                       for some steel)
            sigmamax : float
                       upper limit for the allowed stress,
                       e.g. tensile strength (e.g. 1e8 Pa for aluminium or 4e8 Pa for
                       some steel)
            dichte : float
                     density (e.g. 2.7e3 kg/m^3 for aluminium or 7e3 kg/m^3 for some
                     steel)
            redundanz : integer
                        describes the redundancy. After evaluating the special
                        parameter the other functions are called. If
                        redundanz == 0 the function :py:func:`ttrr` will be
                        called and if redundanz != 0 the
                        function :py:func:`ttrr_redundanz` will be called.
            opt : integer
                  choose the optimization routine: (optional)

                  * 0 glp_simplex (for redundanz == 0) (default)
                  * 1 glp_interior (for redundanz == 0)
                  * 2-5 glp_mip (for redundanz != 0)

                    * 2 do not search greedy-solution
                    * 3 search greedy-solution and go on
                    * 4 search greedy-solution, write greedy solution and go on
                    * 5 search greedy-solution and stop
            ungestoerte_kraefte : numpy.array
                                  If it is a scalar the perturbed forces are
                                  calculated with :py:func:`berechne_gestoerte_kraefte`.
                                  Otherwise kraefte are assumed to be the
                                  perturbed forces. The content of
                                  ungestoerte_kraefte is not used. It will be
                                  saved to the problem data.
            p : float
                is a parameter for the calculation of perturbed forces. (optional)
                see :py:func:`berechne_gestoerte_kraefte`
            art : integer
                  chooses the kind of the perturbation: (optional)
                  see :py:func:`berechne_gestoerte_kraefte`
            sort : integer
                   describes a feature: (optional)

                   * 0 not used (default)
                   * 1 additional constraints will be added to the optimization
                       problem. see :py:func:`ttrr_redundanz`
            zlb : float
                  is a lower bound for the objective function. As we are minimizing
                  the result must not the best solution. Only values greater zero
                  are considered.
            zub : float
                  is a upper bound for the objective function. As we are minimizing
                  the optimization problem may be not solvable. Only values greater
                  zero are considered.
            scale_sigma : float
                  scales sigmamin and sigmamax for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value
            scale_kraefte : float
                  scales kraefte for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value

    copyright + license

    Author: Daniel Mohr.

    Date: 2012-02-19 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2012 Daniel Mohr
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 3 of
     the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, see
     http://www.gnu.org/licenses/"""
    print('###### start ttrr_robust ######')
    if numpy.isscalar(ungestoerte_kraefte):
        print('robust: berechne gestoerte Kraefte')
        ungestoerte_kraefte = kraefte
        kraefte = berechne_gestoerte_kraefte(kraefte,dim,p,art)
    if redundanz != 0:
        [s,w,laengen] = ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=redundanz,opt=opt,ungestoerte_kraefte=ungestoerte_kraefte,sort=sort,zlb=zlb,zub=zub,scale_sigma=scale_sigma,scale_kraefte=scale_kraefte)
    else:
        [s,w,laengen] = ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=redundanz,opt=opt,ungestoerte_kraefte=ungestoerte_kraefte,sort=sort,zlb=zlb,zub=zub,scale_sigma=scale_sigma,scale_kraefte=scale_kraefte)
    print('###### end ttrr_robust ######')
    return [s,w,laengen]

def ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name='test',sigmamin=-1e8,sigmamax=1e8,dichte=2.7e3,redundanz=0,opt=2,ungestoerte_kraefte=0,sort=0,zlb=0,zub=0,scale_sigma=-1.0,scale_kraefte=-1.0):
    """[s,w,exitflag,zeit,laengen] = ttrr_redundanz(dim,knoten,staebe,lager,
                                     kraefte,smax,name='test',sigmamin=-1e8,
                                     sigmamax=1e8,dichte=2.7e3,redundanz=0,
                                     opt=2,ungestoerte_kraefte=0,sort=0,
                                     zlb=0,zub=0,
                                     scale_sigma=-1,scale_kraefte=-1)

        Parameters:

            dim : integer
                  dimension 2 or 3 of the design space
            knoten : numpy.array
                     is a matrix to represent all nodal points with:

                     * knoten(k,:) are the nodal points.
                     * knoten(:,1) are the x-coordinates of the nodal points.
                     * knoten(:,2) are the y-coordinates of the nodal points.
                     * knoten(:,3) are the z-coordinates of the nodal points.
                                   (optional for dim = 2)
            staebe : numpy.array
                     is a vector to represent all trusses:

                     * staebe(s,:) are numbers of the trusses.
                     * staebe(:,1) is the number of the first nodal point.
                     * staebe(:,2) is the number of the second nodal point.
            lager : numpy.array
                    are the numbers of the nodal points as for the coordinate
                    direction which are fixed.
            kraefte : scipy.sparse.lil_matrix
                      are the load forces.

                      kraefte(:,k) are the forces for the k-th load case
            smax : float
                   is the maximal allowed cross section of a bar. If you do not
                   need it, set it to a huge value --- e. g. 1.
                   It is nessecary for the interaction of the integer values
                   and the cross section values (for redundanz != 0).
            name : string
                   is a name for the problem. The result is saved in name.tar.bz2
                   in the current working directory.
            sigmamin : float
                       lower limit for the allowed stress,
                       e.g. compressive strength (e.g. -1e8 Pa for aluminium or -4e8 Pa
                       for some steel)
            sigmamax : float
                       upper limit for the allowed stress,
                       e.g. tensile strength (e.g. 1e8 Pa for aluminium or 4e8 Pa for
                       some steel)
            dichte : float
                     density (e.g. 2.7e3 kg/m^3 for aluminium or 7e3 kg/m^3 for some
                     steel)
            redundanz : integer
                        describes the redundancy. If redundanz == 0 this
                        function is not the right one. :py:func:`ttrr` will be
                        called. A value greater 0 describes a 1/redundanz
                        redundancy --- therefore redundanz >= 2 makes sense. A
                        value lower 0 descirbes a 1-1/redundanz redundancy ---
                        therefore redundanz <= 2 makes sense.
                        Obviosly 1/2 = 1-1/2 are the same redundancy.
                        But the problem and the calculation are different.
                        If there is no error the results are equal.
            opt : integer
                  choose the optimization routine: (optional)

                  * 0 glp_simplex (only for redundanz == 0)
                  * 1 glp_interior (only for redundanz == 0)
                  * 2-5 glp_mip (for redundanz != 0)

                    * 2 do not search greedy-solution (default)
                    * 3 search greedy-solution and go on
                    * 4 search greedy-solution, write greedy solution and go on
                    * 5 search greedy-solution and stop
            ungestoerte_kraefte : numpy.array
                                  is not used. It will be saved to the problem data.
            sort : integer
                   describes a feature: (optional)

                   * 0 not used (default)
                   * 1 additional constraints will be added to the optimization
                       problem. This is only useful for redundanz != 0. If s_i
                       describes a vector of the cross section of the i-th part
                       structure the constraints are l^T s_i <= l^T s_{i+1} for
                       adequate i with l the vector of the lengths of all bars.
            zlb : float
                  is a lower bound for the objective function. As we are minimizing
                  the result must not the best solution. Only values greater zero
                  are considered.
            zub : float
                  is a upper bound for the objective function. As we are minimizing
                  the optimization problem may be not solvable. Only values greater
                  zero are considered.
            scale_sigma : float
                  scales sigmamin and sigmamax for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value
            scale_kraefte : float
                  scales kraefte for the calculation. (optional)
                    * < 0.0 no scaling will be done (default)
                    * = 0.0 autoscaling will be done
                    * > 0.0 scaling will be done with this value

    copyright + license

    Author: Daniel Mohr.

    Date: 2012-02-19 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2012 Daniel Mohr
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 3 of
     the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, see
     http://www.gnu.org/licenses/"""
    if redundanz == 0:
        return ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,sigmamin=sigmamin,sigmamax=sigmamax,dichte=dichte,redundanz=0,opt=opt,ungestoerte_kraefte=ungestoerte_kraefte,sort=sort,zlb=zlb,zub=zub,scale_sigma=scale_sigma,scale_kraefte=scale_kraefte)
    print('###### start ttrr_redundanz ######')
    # schreibe Problem
    tmpdir = schreibe_problem(dim,knoten,staebe,lager,kraefte,smax,name,redundanz,opt,ungestoerte_kraefte,sort,sigmamin,sigmamax,dichte,zlb,zub,scale_sigma,scale_kraefte)
    print('tmpdir: %s' % tmpdir)
    # speichere Problem
    speichere_problem(name,tmpdir)
    # lasse Problem loesen
    lasse_problem_loesen(tmpdir)
    speichere_problem(name,tmpdir)
    # lese Problem
    if redundanz != 0:
        x=lese_int_vektor(tmpdir,"x")
    s=lese_double_vektor(tmpdir,"s")
    w=lese_double_vektor(tmpdir,"w")
    laengen=lese_double_vektor(tmpdir,"laengen")
    for d in os.listdir(tmpdir):
        os.unlink('%s/%s' % (tmpdir,d))
    os.rmdir(tmpdir)
    print('###### end ttrr_redundanz ######')
    return [s,w,laengen]
