#!/usr/bin/python -BO
"""TrussTopologyRobustRedundant

description
===========
ttrr is a software package to do topology optimization of a truss
with respect to robustness and redundancy --- see literatur.

There are 2 parts of ttrr:

* library:
    You can use :py:mod:`ttrr` as a library to import to your own python-code.
    To calculate trusses :py:mod:`ttrr` needs :program:`ttrr_calculate` resp. ttrr_calculate.cc

* command line tool:
    you can use :py:mod:`ttrr_tools.py` as a command line tool.
    for detail::

        ttrr_tools.py -h

dependencies
============
Dependencies for ttrr.py and ttrr_tools.py:
  * Python http://www.python.org/
  * NumPy http://numpy.scipy.org/ http://pypi.python.org/pypi/numpy
  * SciPy http://www.scipy.org/ http://pypi.python.org/pypi/scipy
  
    * and also for NumPy/SciPy: BLAS and LAPACK libraries

Runtime dependencies for ttrr_tools.py to get graphical output (picture, movie,
animation) you need some of the following programs:

  * gnuplot http://www.gnuplot.info/
  * POV-Ray http://www.povray.org/
  * png2yuv from MJPEG http://mjpeg.sourceforge.net/
  * ffmpeg2theora http://www.v2v.cc/~j/ffmpeg2theora/
  * png2theora from libtheora http://www.theora.org
  * FFmpeg http://ffmpeg.org/
  * APNG Assembler http://sourceforge.net/projects/apngasm/
  * convert from ImageMagick http://www.imagemagick.org/

At least for pictures you need gnuplot and/or POV-Ray. The frames for all movie
and animation output are rendered with POV-Ray.

References:
===========

* Mohr, Daniel P. ; Stein, Ina ; Matzies, Thomas ; Knapek, Christina A.:
  Robust Topology Optimization of Truss with regard to Volume.
  In: arXiv - Mathematics, Optimization and Control (2011).
  http://arxiv.org/abs/1109.3782v1

* Mohr, Daniel P.: Redundante Topologieoptimierung.
  Neubiberg, Universitaet der Bundeswehr Muenchen, Fakultaet fuer Luft- und Raumfahrttechnik, Diss., Dezember 2011.
  http://nbn-resolving.de/urn/resolver.pl?urn=urn:nbn:de:bvb:706-2664

copyright + license
===================
Author: Daniel Mohr.

Date: 2012-03-09 (last change).

License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

Copyright (C) 2010, 2011, 2012 Daniel Mohr
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 3 of
 the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, see
 http://www.gnu.org/licenses/
"""

import argparse
#import sys,tempfile,os,struct,array,tarfile,argparse,stat
#import numpy,scipy,scipy.linalg
#from numpy import float64 as typkoordinaten
#from numpy import float64 as typkraefte
#from numpy import float64 as typstabdicken
#from numpy import uint16 as typnummern

from ttrr.intern.ttrr_visualisation import *
from ttrr.intern.ttrr_show_results import *
from ttrr.intern.ttrr_save_data_to import *

def main():
    parser = argparse.ArgumentParser(
        description='ttrr - TrussTopologyRobustRedundant: ttrr is a software package to do topology\noptimization of a truss with respect to robustness and redundancy. For more help\ntype "pydoc ttrr"',
        epilog="Author: Daniel Mohr\nDate: 2012-03-09\nLicense: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007",
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-f',
                        nargs='+',
                        type=str,
                        required=True,
                        dest='file',
                        help='file(s) to process',
                        metavar='file')
    parser.add_argument('--new',
                        action='store_true',
                        default=False,
                        dest='new',
                        help='create only new pictures/movies. do not overwrite existing ones.')
    parser.add_argument('--version',
                        action='version',
                        version='%(prog)s 2012-03-09')
    parser.add_argument('--png_gnuplot',
                        action='store_true',
                        default=False,
                        dest='pngs_gnu',
                        help='create png-pictures with gnuplot')
    parser.add_argument('--pdf_gnuplot',
                        action='store_true',
                        default=False,
                        dest='pdfs_gnu',
                        help='create pdf-pictures with gnuplot')
    parser.add_argument('--gnuplot_cylinder',
                        action='store_true',
                        default=False,
                        dest='gnuplot_cylinder',
                        help='Use gnuplot with creating cylinders as bars. This is slow.')
    parser.add_argument('--png_pov',
                        action='store_true',
                        default=False,
                        dest='pngs_pov',
                        help='create png-pictures with povray')
    parser.add_argument('--ogv',
                        action='store_true',
                        default=False,
                        dest='ogv',
                        help='create ogv-movies with povray and a converter')
    parser.add_argument('-ogv_convert',
                        action='store',
                        default='ffmpeg2theora',
                        type=str,
                        dest='ogv_convert',
                        choices=['ffmpeg2theora','png2theora','ffmpeg'],
                        help='Set the converter to generate the movie from the png-pictures, which povray creates. To create a movie with ffmpeg2theora it needs png2yuv. default: ffmpeg2theora')
    parser.add_argument('--apng',
                        action='store_true',
                        default=False,
                        dest='apng',
                        help='create apng-animation with apngasm')
    parser.add_argument('--mng',
                        action='store_true',
                        default=False,
                        dest='mng',
                        help='create mng-animation with convert')
    parser.add_argument('--gif',
                        action='store_true',
                        default=False,
                        dest='gif',
                        help='create gif-animation with convert')
    parser.add_argument('--yuv',
                        action='store_true',
                        default=False,
                        dest='yuv',
                        help='create YUV4MPEG stream format with png2yuv. This is an uncompressed movie-stream. It is only useful for further conversions.')
    parser.add_argument('--all_bars',
                        action='store_true',
                        default=False,
                        dest='all_bars',
                        help='create pictures with all bars (structural universe)')
    # results
    parser.add_argument('--show_result_table',
                        action='store_true',
                        default=False,
                        dest='show_result_table',
                        help='show results in a table')
    parser.add_argument('--show_results',
                        action='store_true',
                        default=False,
                        dest='show_results',
                        help='show results')
    parser.add_argument('-hist',
                        action='store',
                        default='log_asciiart',
                        type=str,
                        dest='hist',
                        choices=['log_asciiart','numbers','asciiart','all','none'],
                        help='Show in results histogram(s) of bar cross sections of this type. default: log_asciiart')
    parser.add_argument('-ascii_hist_l',
                        nargs=1,
                        default=3,
                        type=int,
                        required=False,
                        dest='ascii_hist_l',
                        help='Set in asciiart histogram the numbers of lines. (default: 3)',
                        metavar='lines')
    parser.add_argument('-ascii_hist_c',
                        nargs=1,
                        default=100,
                        type=int,
                        required=False,
                        dest='ascii_hist_c',
                        help='Set in asciiart histogram the numbers of cols. (default: 100)',
                        metavar='cols')
    # save
    parser.add_argument('--save_module',
                        action='store_true',
                        default=False,
                        dest='save_module',
                        help='save with scipy.io.save_as_module data from file(s) to "{.dat,.dir,.py}"-files')
    parser.add_argument('--save_mat',
                        action='store_true',
                        default=False,
                        dest='save_mat',
                        help='save with scipy.io.savemat data from file(s) to ".mat"-file(s)')
    # grafik parameter
    parser.add_argument('-x',
                        nargs=1,
                        default=1000,
                        type=int,
                        required=False,
                        dest='x',
                        help='x resolution of the pictures (default: 1000)',
                        metavar='pixels')
    parser.add_argument('-y',
                        nargs=1,
                        default=1000,
                        type=int,
                        required=False,
                        dest='y',
                        help='y resolution of the pictures (default: 1000)',
                        metavar='pixels')
    parser.add_argument('-xm',
                        nargs=1,
                        default=800,
                        type=int,
                        required=False,
                        dest='xm',
                        help='x resolution of the movies (default: 800)',
                        metavar='pixels')
    parser.add_argument('-ym',
                        nargs=1,
                        default=600,
                        type=int,
                        required=False,
                        dest='ym',
                        help='y resolution of the movies (default: 600)',
                        metavar='pixels')
    parser.add_argument('-frames',
                        nargs=1,
                        default=250,
                        type=int,
                        required=False,
                        dest='frames',
                        help='the movie will be created from n frames (default: 250)',
                        metavar='n')
    parser.add_argument('-framerate',
                        nargs=1,
                        default=25,
                        type=int,
                        required=False,
                        dest='framerate',
                        help='set the framerate. the movie will be created with n frames per second (default: 25)',
                        metavar='n')
    parser.add_argument('-pov_angle',
                        nargs=1,
                        default=33,
                        type=int,
                        required=False,
                        dest='pov_angle',
                        help='For rendering pictures by povray the view angle is set to a (default: 33)',
                        metavar='a')
    parser.add_argument('-pov_angle_m',
                        nargs=1,
                        default=40,
                        type=int,
                        required=False,
                        dest='pov_angle_m',
                        help='For rendering animations by povray the view angle is set to a (default: 40)',
                        metavar='a')
    parser.add_argument('-minfact',
                        nargs=1,
                        default=1e-6,
                        type=float,
                        required=False,
                        dest='minfaktor',
                        help='a truss with a cross section less than mf*max(s) is ignored. max(s) is the maximum over all truss cross sections. if mf=0 no truss is ignored. (default: 1e-6)',
                        metavar='mf')
    args = parser.parse_args()
    if args.pngs_gnu or args.pdfs_gnu or args.pngs_pov or args.ogv or args.apng or args.mng or args.gif or args.yuv:
        visualisation(args)
    if args.show_results:
        show_results(args)
    if args.show_result_table:
        show_result_table(args)
    if args.save_mat or args.save_module:
        save_datas_to(args)

if __name__ == "__main__":
    main()
